package orm.generators;

import java.io.*;
import java.util.*;
import java.nio.channels.*;

public class JavaCompiler {
	
	
	private String message= "";
	public JavaCompiler(){}
	
	public boolean CompileFile(File f) {
		return CompileFile(f, null);
	}
	
	public boolean CompileFile(File f, String classPath) {
		//check file properties
		if (!f.exists()) {
			message = f.getAbsolutePath() + " does not exists";
			return false;
		}
		if (!f.isFile()) {
			message = f.getAbsolutePath() + " is not a file";
			return false;
		}
		if (!f.getName().toLowerCase().endsWith(".java")){
			message = f.getAbsolutePath() + " is not a java source file";
			return false;
		}
		//end check
		
		ArrayList<String> args = new ArrayList<String>();
		args.add(System.getenv("JAVA_HOME")+ "/bin/javac");
		args.add(f.getAbsolutePath());
		if (classPath != null) {
			args.add("-classpath");
			args.add(classPath);
		}
		
		return runProcess(args.toArray(new String[args.size()]));
	}
	
	public boolean CompileJar(File dir, String classPath,  File jarFile) {
		return CompileJar(dir, classPath, jarFile.getName(), jarFile.getParentFile());
	}
	public boolean CompileJar(File dir, String classPath, String jarName, File jarDirectory) {
		
		if (!dir.exists()) {
			message = dir.getAbsolutePath() + " does not exists";
			return false;
		}
		if (!dir.isDirectory()) {
			message = dir.getAbsolutePath() + " is not a file";
			return false;
		}
		if (!jarDirectory.isDirectory() && !jarDirectory.mkdirs()) {
			message = "Could not create "  + jarDirectory.getAbsolutePath();
			return false;
		}
		
		if (!jarDirectory.isDirectory()) {
			message = jarDirectory.getAbsolutePath() + " is not a directory";
			return false;
		}
		
		//end check
		ArrayList<String> args = new ArrayList<String>();
		args.add(System.getenv("JAVA_HOME")+ "/bin/javac");
		args.add(dir.getAbsolutePath() + "\\*java");
		args.add("-sourcepath");
		args.add(dir.getAbsolutePath());
		args.add("-classpath");
		args.add(classPath);
		boolean isOK = runProcess(args.toArray(new String[args.size()]));
		orm.runtime.ORMClassLoader.getLoader().loadClasses(dir);
		isOK = isOK && generateJar(jarName, dir);
		return isOK;
	}
	
	
	private boolean generateJar(String jarName, File directory) {
		if (!jarName.endsWith(".jar")) {
			message = jarName + " is not a valid jar name";
			return false;
		}
		ArrayList<String> args = new ArrayList<String>();
		args.add(System.getenv("JAVA_HOME")+ "/bin/jar");
		File jarFile = new File(directory,jarName);
		args.add((jarFile.exists() ? "u" : "c") + "f" );
		args.add(jarFile.getAbsolutePath());
		args.add(directory.getAbsolutePath() + "/*.class");
		
		return runProcess(args.toArray(new String[args.size()]));
	}
	public String getMessage() {
		return message;
	}
	private boolean runProcess(String[] args) {
	    message = "";
	    try {
			Process p = Runtime.getRuntime().exec(args);
			StringBuffer stdIn = new StringBuffer();
			StringBuffer stdErr = new StringBuffer();
			InputStreamListener stdInListener = new InputStreamListener(p.getInputStream(),stdIn);
			InputStreamListener stdErrListener = new InputStreamListener(p.getErrorStream(),stdErr);
			stdInListener.start();
			stdErrListener.start();
			int result = p.waitFor();
			stdInListener.join();
			stdErrListener.join();
			if (stdIn.length() > 0) {
				message += "\nOutput: " + stdIn.toString();
			}
			if (stdErr.length() > 0) {
				message += "\nOutput: " + stdErr.toString();
			}
			return result == 0;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}
		return false;
		
	}
	
	//inner classes
		private class InputStreamListener  extends Thread {

			private InputStream stream;
			private StringBuffer message;
			
			public InputStreamListener(InputStream streamToGet, StringBuffer messageBuffer) {
				stream = streamToGet;
				message = messageBuffer;
			}
			/* (non-Javadoc)
			 * @see java.lang.Thread#run()
			 */
			public void run() {
				BufferedReader br = null;
				try {
					br = new BufferedReader(new InputStreamReader(stream));
					String line = null;
					while ((line = br.readLine()) != null) {
						message.append(line).append("\n");
					}
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					try {
						br.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
	//end innerclasses
}
