package orm.runtime;

import java.io.*;
import java.nio.channels.FileChannel;
import java.security.*;

public class ORMClassLoader extends ClassLoader {

	private static final ORMClassLoader loader = new ORMClassLoader();
	
	public static ORMClassLoader getLoader() {
		return loader;
	}
	
	private ORMClassLoader() {
		super(ClassLoader.getSystemClassLoader());
	}
	
	public void loadClass(File classFile) {
		if (!classFile.getName().endsWith(".class")) return;
		
		FileInputStream fis;
		try {
			fis = new FileInputStream(classFile);
			try {
				java.nio.MappedByteBuffer buff = fis.getChannel().map(FileChannel.MapMode.READ_ONLY, 0,
						classFile.length());
				String className = classFile.getName().substring(0, classFile.getName().length() - ".class".length());
				super.resolveClass(super.defineClass("orm.impl." + className, buff, null));
				
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				fis.close();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	public void loadClasses(File dir) {
		if (!dir.isDirectory()) loadClass(dir);
		
		for (File cFile : dir.listFiles(new FilenameFilter () { public boolean accept (File dir, String fileName) {return fileName.endsWith(".class");}})) {
			loadClass(cFile);
		}
	}
}
