package orm.structure;

import java.lang.reflect.*;
import orm.api.*;

public class FieldStructure implements IFieldStructure{
	protected String name = null;
	protected EntityStructure owner = null;
	protected Field field = null;
	
	/**
	 * @return the name
	 */
	public String getFieldName() {
		return name;
	}
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return the owner
	 */
	public EntityStructure getOwner() {
		return owner;
	}
	/**
	 * @param owner the owner to set
	 */
	public void setOwner(EntityStructure owner) {
		this.owner = owner;
	}
	
	public Class getFieldType() {
		return field.getType();
	}
	
	/**
	 * @return the type
	 */
	public Field getField() {
		return field;
	}
	/**
	 * @param type the type to set
	 */
	public void setField(Field field) {
		this.field = field;
	}
	
	public static FieldStructure getColumn(Field f) {
		int modifiers = f.getModifiers();
		OrmPersistent persistentAnnotation = f.getAnnotation(OrmPersistent.class);
		
		if(Modifier.isPrivate(modifiers) || Modifier.isProtected(modifiers)){
			if (persistentAnnotation != null && persistentAnnotation.persistent() == false) {
				return null;
			} 
		} else {
			if (persistentAnnotation == null || persistentAnnotation.persistent() == false) {
				return null;
			}
		}
		FieldStructure res = new FieldStructure();
		res.setField(f);
		if (f.getAnnotation(OrmName.class) != null) {
			res.setName(f.getAnnotation(OrmName.class).Name());
		} else {
			res.setName(f.getName());
		}
		return res;
	}
	
	public boolean hasLateLoad() {
		return field.getAnnotation(OrmPersistent.class).lateLoad();
	}
	
}
