package sql.sqlserver;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.PriorityQueue;

import orm.api.IOrmDomain;
import orm.api.IOrmEntity;
import orm.api.IDataStructure;
import orm.api.IEntityStructure;
import orm.structure.StartUpType;

public class SqlServerDomain implements IOrmDomain {
	private IDataStructure structure;
	Hashtable<String,SqlTable> structureMapping = new Hashtable<String,SqlTable>();
	Hashtable<Class,String> classMapping = new Hashtable<Class,String>();
	
	
	static final String primaryOrmTableName = "orm_primary";
	
	public void start(StartUpType type, orm.api.IDataStructure structure) throws SqlServerException {
		this.structure = structure;
		init();
		switch(type) {
			case Create:
				internal_createSQL();
				internal_start();
				break;
			case Update:
			default:
				internal_updateSQL();
				internal_start();
				break;
			case NoAction:
				internal_start();
				break;
		}
	}

	public boolean commit(IOrmEntity entity) {
		// TODO Auto-generated method stub
		return false;
	}

	public IOrmEntity getNewObject(Class c) {
		// TODO Auto-generated method stub
		return null;
	}

	public IOrmEntity getObject(Class c, String id) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getOrmID(IOrmEntity entity) {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean lock(IOrmEntity entity) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean rollback(IOrmEntity entity) {
		// TODO Auto-generated method stub
		return false;
	}
	
	private void init() throws SqlServerException
	{
		for(IEntityStructure eStructure : structure.getEntityStructures()) {
			classMapping.put(eStructure.getEntityClass(), eStructure.getName());
			SqlTable t = new SqlTable(eStructure, this);
			structureMapping.put(eStructure.getName(), t);
		}
		for(IEntityStructure eStructure : structure.getEntityStructures()) {
			IEntityStructure baseEStructure = eStructure.getBaseEntityStructure();
			if (baseEStructure != null) {
				SqlTable t = structureMapping.get(eStructure.getName());
				SqlTable baseTable = structureMapping.get(baseEStructure.getName());
				t.setBaseTable(baseTable);
			}
		}
	}
	
	private void internal_createSQL() throws SqlServerException
	{
		StringBuffer buffer = new StringBuffer();
		PriorityQueue<SqlTable> tables = new PriorityQueue<SqlTable> (
				structureMapping.size(), 
				new Comparator<SqlTable> () {
					public int compare(SqlTable arg0,SqlTable arg1) {
						int dep1 = arg0.getDepth();
						int dep2 = arg1.getDepth();
						if (dep1 < dep2) return -1;
						if (dep1 > dep2) return  1;
						return 0;
					}	
		});
		
		//get the tables in ordered queue
		for(IEntityStructure eStructure : structure.getEntityStructures()) {
			tables.add(this.structureMapping.get(eStructure.getName()));
		}
		
		buffer.append("\n CREATE TABLE " + primaryOrmTableName + " \n ( \n id bigint NOT NULL IDENTITY (1, 1), \n class_name varchar(200) NOT NULL \n )  ON [PRIMARY] \n GO");
		buffer.append("\n ALTER TABLE " + primaryOrmTableName + " ADD CONSTRAINT \n PK_" + primaryOrmTableName+ " PRIMARY KEY CLUSTERED (id) ON [PRIMARY]\nGO\n");
		
		for(SqlTable t : tables) {
			buffer.append(t.getCreateSqlScript());
		}
		for(SqlTable t : tables) {
			buffer.append(t.getCreateRelationSqlScript());
		}
		
		System.out.println(buffer);
	}
	private void internal_updateSQL() 
	{
	
	}
	
	private void internal_start() throws SqlServerException
	{
		
	}
	
	public SqlTable getSqlTableByClass(Class c) {
		String name = this.classMapping.get(c);
		if (name == null) {
			return null;
		}
		else {
			return this.structureMapping.get(name);
		}
	}
}
