package sql.sqlserver;

import java.util.ArrayList;

import orm.api.IFieldStructure;
import orm.api.relations.IRelation;
import orm.api.IEntityStructure;
import orm.api.IFieldStructure;

public class SqlTable {
	private IEntityStructure entityStructure;
	ArrayList<SqlColumn> columns = new ArrayList<SqlColumn>();
	ArrayList<ISqlRelation> relations = new ArrayList<ISqlRelation>();
	private SqlServerDomain domain;
	
	private SqlTable baseTable = null;
	
	public SqlTable(IEntityStructure entityStructure, SqlServerDomain domain) throws SqlServerException {
		this.entityStructure = entityStructure;
		this.domain = domain;
		for(IFieldStructure fStructure : entityStructure.getFields()) {
			columns.add(new SqlColumn(fStructure));
		}
		for(IRelation rel : entityStructure.getRelations()) {
			ISqlRelation sqlRel = SqlRelationFactory.getRelation(this, rel);
			if (sqlRel != null) {
				relations.add(sqlRel);
			}
		}
	}
	
	public String getName () {
		return this.entityStructure.getName();
	}
	public String getSqlName () {
		return "[" + getName() + "]";
	}
	String getSqlTriggerOnInsertName() {
		return "[trgInstead_"  + getName() + "]";
	}

	SqlTable getBaseTable() {
		return baseTable;
	}

	void setBaseTable(SqlTable baseTable) {
		this.baseTable = baseTable;
	}
	
	private Integer depth = null;
	public int getDepth() {
		if (depth == null) {
			int tmpDepth = this.baseTable != null ? 1 + this.baseTable.getDepth() : 0;
			depth =  new Integer(tmpDepth);
		}
		return depth.intValue();
	}
	
	String getCreateSqlScript() {
		StringBuffer res = new StringBuffer("\nCreate Table ");
		res.append(getSqlName()).append("(\n");
		res.append("id bigint NOT NULL");
		for(SqlColumn c : this.columns) {
			res.append(",\n").append(c.getSqlName()).append(" ").append(c.getColumnType().getSql());
		}
		res.append("\n) ON [PRIMARY] \nGO\n");
		
		//add primary key
		res.append("ALTER TABLE ")
		.append(getSqlName())
		.append(" ADD CONSTRAINT PK_")
		.append(getName())
		.append(" PRIMARY KEY CLUSTERED (id) ON [PRIMARY] \nGO\n");
	
		
		if (this.baseTable != null) {
			res.append("ALTER TABLE ")
			.append(getSqlName())
			.append("  ADD CONSTRAINT ")
			.append("FK_")
			.append(this.baseTable.getName())
			.append("_")
			.append(getName())
			.append(" FOREIGN KEY (id) REFERENCES ")
			.append(this.baseTable.getName())
			.append("(id) ON UPDATE CASCADE ON DELETE CASCADE\nGO\n");
		} else {
//			foregn key orm_base
			res.append("ALTER TABLE ")
				.append(getSqlName())
				.append("  ADD CONSTRAINT ")
				.append("FK_")
				.append(SqlServerDomain.primaryOrmTableName)
				.append("_")
				.append(getName())
				.append(" FOREIGN KEY (id) REFERENCES ")
				.append(SqlServerDomain.primaryOrmTableName)
				.append("(id) ON UPDATE CASCADE ON DELETE CASCADE\nGO\n");
		}
		return res.toString();
	}
	
	String getCreateRelationSqlScript() throws SqlServerException {
		StringBuffer res = new StringBuffer();
		for(ISqlRelation rel : this.relations) {
			res.append(rel.getCreateSqlScript());
		}
		return res.toString();
	}
	
	public SqlServerDomain getDomain() {
		return domain;
	}
	
	public ISqlBiDirectRelation getBiDirectRelationByName(String relName) {
		for(ISqlRelation rel : this.relations) {
			if (rel instanceof ISqlBiDirectRelation) {
				ISqlBiDirectRelation current = (ISqlBiDirectRelation) rel;
				if (current.getName().equalsIgnoreCase(relName)) {
					return current;
				}
			}
		}
		return null;
	}
	
	public SqlColumn[] getColumns() {
		return columns.toArray(new SqlColumn[columns.size()]);
	}
}
