package sql.sqlserver.relations;

import orm.api.relations.IBiDirectRelation;
import orm.api.relations.IRelation;
import sql.sqlserver.ISqlBiDirectRelation;
import sql.sqlserver.SqlServerException;
import sql.sqlserver.SqlTable;

public class SqlRelationOneToOneBiDirect extends SqlRelationOneToOne implements ISqlBiDirectRelation{

	
	public SqlRelationOneToOneBiDirect(SqlTable table, IBiDirectRelation rel) {
		super(table, rel);
	}
	
	public String getName() {
		return ((IBiDirectRelation)super.rel).getRelationName();
	}

	@Override
	public String getCreateSqlScript() throws SqlServerException{
		SqlTable referredTable = this.getTable().getDomain().getSqlTableByClass(rel.getTargetClass());
		ISqlBiDirectRelation refRel = referredTable.getBiDirectRelationByName(this.getName());

		if (refRel ==  null) throw new SqlServerException("referred bidirect relation does not exists");
		
		if (refRel.isCreated()) return "";
		
		//add column_id for the first table
		StringBuffer res = new StringBuffer(super.getCreateSqlScript_CreateNewColumnID());
		res.append(((SqlRelationOneToOneBiDirect)refRel).getCreateSqlScript_CreateNewColumnID());
		
		
		String column1 = getRelColumnName();
		String column2 = refRel.getRelColumnName();
		
//		res.append("CREATE UNIQUE NONCLUSTERED INDEX IX_")
//		.append(table.getName())
//		.append("_")
//		.append(column1)
//		.append(" ON ")
//		.append(table.getSqlName())
//		.append(" (")
//		.append(column1)
//		.append("_id")
//		.append(") ON [PRIMARY] \nGO\n");
//		
//		res.append("CREATE UNIQUE NONCLUSTERED INDEX IX_")
//		.append(referredTable.getName())
//		.append("_")
//		.append(column2)
//		.append(" ON ")
//		.append(referredTable.getSqlName())
//		.append(" (")
//		.append(column2)
//		.append("_id")
//		.append(") ON [PRIMARY] \nGO\n");

		//add foreign keys
		res.append("ALTER TABLE ")
		.append(table.getSqlName())
		.append(" ADD CONSTRAINT FK_")
		.append(column1)
		.append("_to_")
		.append(referredTable.getName())
		.append("_")
		.append(column2)
		.append(" FOREIGN KEY (")
		.append(column1)
		.append("_id")
		.append(") REFERENCES ")
		.append(referredTable.getSqlName())
		.append(" (")
		.append("id")
		.append(") ON DELETE NO ACTION ON UPDATE NO ACTION\nGO\n");
		
		res.append("ALTER TABLE ")
		.append(referredTable.getSqlName())
		.append(" ADD CONSTRAINT FK_")
		.append(column2)
		.append("_to_")
		.append(table.getName())
		.append("_")
		.append(column1)
		.append("_2 FOREIGN KEY (")
		.append(column2)
		.append("_id")
		.append(") REFERENCES ")
		.append(table.getSqlName())
		.append(" (")
		.append("id")
		.append(") ON DELETE NO ACTION ON UPDATE NO ACTION\nGO\n");
		
		is_created = true;
		return res.toString();
	}

	@Override
	public boolean isBiDirect() {
		return true;
	}
	
	private boolean is_created = false;
	public boolean isCreated() {
		return is_created;
	}

	
}
