package orm.runtime;

import orm.structure.*;
import java.util.*;
import orm.api.*;

public class OrmRuntime {
	
	DataStructure structure = null;
	ClassStructure classStructure = null;
	IOrmDomain domain;
	StartUpType type;
	
	static Hashtable<String,OrmRuntime> ormInstances = new Hashtable<String,OrmRuntime>();
	
	private OrmRuntime(Class[] classes, StartUpType type,IOrmDomain domain) throws ORMException {
		structure = DataStructure.getStructure(classes);
		classStructure = ClassStructure.getStructure(structure, type);
		this.domain = domain;
		this.type = type;
	}
	
	public static OrmRuntime getOrmInstance(Class[] classes,StartUpType type, String name, IOrmDomain domain) throws ORMException {
		OrmRuntime res =  new OrmRuntime(classes, type, domain);
		ormInstances.put(name, res);
		return res;
	}
	
	public static void commit(IOrmPersistent obj) {
		for(OrmRuntime orm : ormInstances.values()) {
			orm.commitInternal(obj);
		}
	}
	
	
	private void commitInternal(IOrmPersistent obj) {
		domain.commit(obj.getOrmEntityWithUpdatedData());
	}
	
	public void start() throws ORMException {
		this.domain.start(type, structure);
	}
	
	public static IOrmPersistent createObject(Class c,String ormDomain, String ormId) {
		OrmRuntime runtime = ormInstances.get(ormDomain);
		if (!runtime.structure.containsClass(c)) {
			return null;
		}
		IOrmEntity entity = runtime.domain.getObject(c, ormId);
		try {
			return (IOrmPersistent) c.getConstructor(new Class[] {IOrmEntity.class}).newInstance(entity);
		} catch (Exception e) {
			return null;
		} 
	}
	public static IOrmPersistent createNewObject(Class c,String ormDomain) {
		OrmRuntime runtime = ormInstances.get(ormDomain);
		if (!runtime.structure.containsClass(c)) {
			return null;
		}
		IOrmEntity entity = runtime.domain.getNewObject(c);
		try {
			return (IOrmPersistent) c.getConstructor(new Class[] {IOrmEntity.class}).newInstance(entity);
		} catch (Exception e) {
			return null;
		} 
	}
	
	
}
