/*
 * Created on Sep 5, 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package orm.structure;

import java.util.*;
import java.lang.reflect.*;
import orm.api.*;
import orm.api.relations.IRelation;

/**
 * @author I040151
 *
 */
public class EntityStructure implements IEntityStructure {
	protected String name =null;
	protected DataStructure structure = null;
	protected Class<?> classObject;
	
	protected EntityStructure baseEntityStructure = null;
	protected ArrayList<EntityStructure> derivedEntityStructures = new ArrayList<EntityStructure>();
	
	
	private ArrayList<FieldStructure> columns = new ArrayList<FieldStructure>();
	private ArrayList<IRelation> relations = new ArrayList<IRelation>();
	
	public EntityStructure(DataStructure structure) {
		this.structure= structure;
	}
	
	
	/**
	 * @return the baseEntityStructure
	 */
	public EntityStructure getBaseEntityStructure() {
		return baseEntityStructure;
	}
	
	public ArrayList<EntityStructure> getDerivedEntityStructures() {
		return derivedEntityStructures;
	}


	/**
	 * @param baseEntityStructure the baseEntityStructure to set
	 */
	public void setBaseEntityStructure(EntityStructure baseEntityStructure) {
		this.baseEntityStructure = baseEntityStructure;
		baseEntityStructure.derivedEntityStructures.add(this);
	}


	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the structure.
	 */
	public DataStructure getStructure() {
		return structure;
	}
	/**
	 * @param structure The structure to set.
	 */
	public void setStructure(DataStructure structure) {
		this.structure = structure;
	}
	
	public synchronized void addField(FieldStructure c) {
		columns.add(c);
	}
	
	//public FieldStructure[] getFields() {
	//	return columns.toArray(new FieldStructure[columns.size()]);
	//}
	public ArrayList<? extends IFieldStructure> getAllFields() {
		if (this.baseEntityStructure == null) {
			return getFields();
		} else {
			ArrayList<IFieldStructure> res = new ArrayList<IFieldStructure>(getFields());
			res.addAll(this.baseEntityStructure.getAllFields());
			return res;
		}
	}
	
	public ArrayList<? extends IFieldStructure> getFields() {
		return columns;
	}
	public ArrayList<? extends IRelation> getRelations() {
		return relations;
	}
	
	public Class getEntityClass() {
		return classObject;
	}
	public void setEntityClass(Class c) {
		classObject = c;
	}
	
	public static  EntityStructure getEntityStructure(Class<?> c, DataStructure structure){
		
		EntityStructure res = new EntityStructure(structure);
		if (c.getAnnotation(OrmName.class) == null) {
			//use default name
			res.setName(c.getSimpleName());
		} else {
			res.setName(c.getAnnotation(OrmName.class).Name());
		}
		res.setEntityClass(c);
		for(Field f : getFields(c)){
			
			IRelation rel = res.getRelation(f);
			if (rel != null) {
				res.relations.add(rel);
			} else {
				FieldStructure col = FieldStructure.getColumn(f);
				if (col!=null) res.addField(col);
			}
		}
		return res;
	}
	private static LinkedList<Field> getFields(Class c) {
		LinkedList<Field> res = new LinkedList<Field>();
		res.addAll(Arrays.asList(c.getDeclaredFields()));
		return res;
	}
	private static LinkedList<Field> getAllFields (Class c) {
		LinkedList<Field> res = new LinkedList<Field>();
		while(!c.equals(Object.class)) {
			res.addAll(Arrays.asList(c.getDeclaredFields()));
			c = c.getSuperclass();
		}
		return res;
	}
	
	private boolean isRelation(Field f) {
		int modifiers = f.getModifiers();
		OrmPersistent persistentAnnotation = f.getAnnotation(OrmPersistent.class);
		
		if(Modifier.isPrivate(modifiers) || Modifier.isProtected(modifiers)){
			if (persistentAnnotation != null && persistentAnnotation.persistent() == false) {
				return false;
			} 
		} else {
			if (persistentAnnotation == null || persistentAnnotation.persistent() == false) {
				return false;
			}
		}
		
		Class<?> fieldType = f.getType();
		//Many to one or Many to Many relations
		if (fieldType.isArray()) return true;
		if (fieldType.isEnum()) return true;
		if (fieldType.isAssignableFrom(List.class)) return true;
		if (fieldType.isAssignableFrom(Collection.class)) return true;
		//One to One or Many to One relations
		if (structure.containsClass(fieldType)) return true;
		return false;
	}
	
	private IRelation getRelation(Field f) {
		if (!isRelation(f)) return null;
		f.setAccessible(true);
		OrmRelation  relation = f.getAnnotation(OrmRelation.class);
		return relation.relationType().getRelation(f);
	}
	
	
	
}
